import Sofa

import os
path = os.path.dirname(os.path.abspath(__file__))+'/mesh/'

###########################################################################
# Changes only inside of blocks marked like this.
from splib import animation

# Settings I used to generate the videos:
# DT = 0.01, RAMP_UP_PERIOD = 0.2
# DT = 0.001, RAMP_UP_PERIOD = 0.2
# DT = 0.0001, RAMP_UP_PERIOD = 0.2
# DT = 0.0001, RAMP_UP_PERIOD = 1.0
DT = 0.0001
DURATION = 1.0
RAMP_UP_PERIOD = 0.2
MAX_PRESSURE = 4.0

def anim_fn(cavity, factor):
    t = factor * DURATION
    cavity.SurfacePressureConstraint.value = min(
        MAX_PRESSURE, t * MAX_PRESSURE / RAMP_UP_PERIOD
    )

# End of changes (More changes below to set dt and use pressure control)
###########################################################################



def createScene(rootNode):

                rootNode.createObject('RequiredPlugin', pluginName='SoftRobots SofaOpenglVisual SofaSparseSolver SofaPreconditioner')
                rootNode.createObject('VisualStyle', displayFlags='showVisualModels hideBehaviorModels showCollisionModels hideBoundingCollisionModels hideForceFields showInteractionForceFields hideWireframe')

                rootNode.createObject('FreeMotionAnimationLoop')
                rootNode.createObject('GenericConstraintSolver', maxIterations=100, tolerance = 0.0000001)
                #############################################################
                # Set dt
                rootNode.findData('dt').value = DT
                #############################################################

		#bunny
                bunny = rootNode.createChild('bunny')
                bunny.createObject('EulerImplicitSolver', name='odesolver')
                bunny.createObject('ShewchukPCGLinearSolver', iterations=15, name='linearsolver', tolerance=1e-5, preconditioners='preconditioner', use_precond=True, update_step=1)

                bunny.createObject('MeshVTKLoader', name='loader', filename=path+'Hollow_Stanford_Bunny.vtu')
                bunny.createObject('TetrahedronSetTopologyContainer', src='@loader', name='container')
                bunny.createObject('TetrahedronSetTopologyModifier')

                bunny.createObject('MechanicalObject', name='tetras', template='Vec3', showIndices=False)
                bunny.createObject('UniformMass', totalMass=0.5)
                bunny.createObject('TetrahedronFEMForceField', template='Vec3', name='FEM', method='large', poissonRatio=0.3,  youngModulus=18000)

                bunny.createObject('BoxROI', name='boxROI', box=[-5, -15, -5,  5, -4.5, 5], drawBoxes=True, position="@tetras.rest_position", tetrahedra="@container.tetrahedra")
                bunny.createObject('RestShapeSpringsForceField', points='@boxROI.indices', stiffness=1e12)

                bunny.createObject('SparseLDLSolver', name='preconditioner')
                bunny.createObject('LinearSolverConstraintCorrection', solverName='preconditioner')
                #bunny.createObject('UncoupledConstraintCorrection')


		#bunny/cavity
                cavity = bunny.createChild('cavity')
                cavity.createObject('MeshObjLoader', name='loader', filename=path+'Hollow_Bunny_Body_Cavity.obj')
                cavity.createObject('MeshTopology', src='@loader', name='topo')
                cavity.createObject('MechanicalObject', name='cavity')
                ###############################################################
                # Change the constraint to pressure control with 0 initial pressure
                cavity.createObject('SurfacePressureConstraint', triangles='@topo.triangles', value=0, valueType='pressure', maxPressureVariation=DT*MAX_PRESSURE/RAMP_UP_PERIOD)
                ###############################################################
                cavity.createObject('BarycentricMapping', name='mapping',  mapForces=False, mapMasses=False)


		#bunny/bunnyVisu
                bunnyVisu = bunny.createChild('visu')
                bunnyVisu.createObject('TriangleSetTopologyContainer', name='container')
                bunnyVisu.createObject('TriangleSetTopologyModifier')
                bunnyVisu.createObject('Tetra2TriangleTopologicalMapping', name='Mapping', input="@../container", output="@container")

                bunnyVisu.createObject('OglModel', color=[0.3, 0.2, 0.2, 0.6])
                bunnyVisu.createObject('IdentityMapping')

                ############################################################
                # Add animation fn
                animation.AnimationManager(rootNode)
                animation.animate(anim_fn, {'cavity': cavity},
                                  duration=DURATION)
                ############################################################

                return rootNode
